package com.freshcodes.red5pro;

import android.content.Context;
import android.view.View;

import androidx.annotation.NonNull;

import com.red5pro.streaming.R5Connection;
import com.red5pro.streaming.R5Stream;
import com.red5pro.streaming.config.R5Configuration;
import com.red5pro.streaming.view.R5VideoView;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class Red5StreamPlayerView implements PlatformView {
    @NonNull
    private final R5Configuration configuration;
    String streamName;

    R5VideoView videoView;
    protected R5Stream stream;

    Red5StreamPlayerView(@NonNull final Context context, BinaryMessenger messenger, int id, @NonNull R5Configuration configuration, @NonNull String streamName) {
        this.configuration = configuration;
        this.streamName = streamName;
        videoView = new R5VideoView(context);
        stream = new R5Stream(new R5Connection(configuration));
        videoView.attachStream(stream);

        MethodChannel methodChannel = new MethodChannel(messenger, "Red5StreamPlayerView/" + id);
        methodChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler() {
            @Override
            public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                switch (call.method) {
                    case "play":
                        play();
                        result.success(true);
                        break;
                    case "stop":
                        stop();
                        result.success(true);
                        break;
                    default:
                        result.notImplemented();
                        break;
                }
            }
        });
    }

    @Override
    public View getView() {
        return videoView;
    }

    @Override
    public void dispose() {
        stop();
    }

    public void play() {
        if (stream != null) {
            stream.play(streamName);
        }
    }

    public void stop() {
        if (stream != null) {
            stream.stop();
        }
    }
}
